@@ -6,11 +6,13 @@ import logging
 
 log = logging.getLogger(__name__)
 
+
 class ErrorCodes(Enum):
     FETCH_ORDER_STATUS_ERROR = 'Error while fetching fill statuses'
     INSUFFICIENT_FUNDS = 'Insufficient funds'
     FILL_REQUEST_SEQUENCE_ERROR = 'Fill request sequence error'
 
+
 class Accountant:
     """
     Accountant binds together 2 concepts - BankAccount & Ledger
@@ -191,41 +193,26 @@ class Accountant:
         log.info(new_books)
         return new_books
 
-    def block_for_order(self, order_id: str, order_type: str, from_currency: str, to_currency: str, from_amount: str, to_amount: str, memo: str = None, metadata: dict = {}):
+    def block_for_order(self, order_id: str, order_type: str, currency: str, value: str, metadata: dict = {}):
         log.info("---------Block_Limit_Order_Funds----------")
 
-        memo = order_id+"_BLOCK_ORDER" if not memo else memo + "_BLOCK_ORDER"
+        memo = f"{order_id}_BLOCK_ORDER"
 
         metadata['order_id'] = order_id
         metadata['order_type'] = order_type
         metadata['operation'] = 'ORDER'
-        metadata['from_currency'] = from_currency
-        metadata['to_currency'] = to_currency
+        metadata['from_currency'] = currency
         metadata['type'] = 'BLOCK'
-        operation_type = OperationType.CUSTOM
-        entries = [
-            {
-                "bookId": self.bank_account['ledger_books']['main_balance']['id'],
-                "assetId": from_currency,
-                "value": str(Decimal(from_amount) * Decimal("-1"))
-            },
-            {
-                "bookId": self.bank_account['ledger_books']['blocked_balance']['id'],
-                "assetId": from_currency,
-                "value":  from_amount
-            },
-            {
-                "bookId": self.bank_account['ledger_books']['blocked_balance']['id'],
-                "assetId": to_currency,
-                "value": to_amount
-            },
-            {
-                "bookId": self.DEFAULT_CASH_BOOK_ID['id'],
-                "assetId": to_currency,
-                "value": str(Decimal(to_amount) * Decimal("-1"))
-            }
-        ]
-        operation = self.ledger.post_operation(operation_type, entries, memo, metadata)
+
+        operation = self.ledger.post_transfer(
+            from_book_id=self.bank_account['ledger_books']['main_balance']['id'],
+            to_book_id=self.bank_account['ledger_books']['blocked_balance']['id'],
+            asset_id=currency,
+            value=value,
+            memo=memo,
+            metadata=metadata
+        )
+
         log.info(operation)
         return operation
 
@@ -241,122 +228,112 @@ class Accountant:
         balance_list = self.ledger.get_book_balances_bulk(query)
         return balance_list
 
-    def unblock_for_order(self, order_id: str, order_type: str, memo: str = None, metadata: dict = {}):
+    def unblock_for_order(self, order_id: str, order_type: str, metadata: dict = {}):
         log.info("---------Unblock_Limit_Order_Funds----------")
 
-        memo = memo+'_UNBLOCK_ORDER' if memo else order_id+'_UNBLOCK_ORDER'
+        memo = f"{order_id}_UNBLOCK_ORDER"
 
-        [block_book_order_balances, main_book_order_balances, cash_book_order_balances] =\
-            self.get_book_balances_bulk([{'book_id':self.bank_account['ledger_books']['blocked_balance']['id'], 'metadata_filter':{'order_id':order_id}},
-                                         {'book_id':self.bank_account['ledger_books']['main_balance']['id'], 'metadata_filter':{'order_id':order_id}},
-                                         {'book_id': self.DEFAULT_CASH_BOOK_ID['id'], 'metadata_filter': {'order_id': order_id}}])
-
-        from_currency = None
-        to_currency = None
-        if not block_book_order_balances:
-            raise Exception(ErrorCodes.INSUFFICIENT_FUNDS.value)
-        for (key,val) in main_book_order_balances.items():
-            if Decimal(val)<0:
-                from_currency = key
-                break
-        for (key,val) in cash_book_order_balances.items():
-            if Decimal(val)<0:
-                to_currency = key
-                break
-        from_amount = block_book_order_balances.get(from_currency,'0')
-        to_amount = block_book_order_balances.get(to_currency,'0')
-        if not ((from_currency and to_currency) and (Decimal(from_amount)>0 or Decimal(to_amount)>0)):
-            raise Exception(ErrorCodes.INSUFFICIENT_FUNDS.value)
-
-        operation_type = OperationType.CUSTOM
-        entries = [
-            {
-                "bookId": self.bank_account['ledger_books']['main_balance']['id'],
-                "assetId": from_currency,
-                "value": from_amount
-            },
-            {
-                "bookId": self.bank_account['ledger_books']['blocked_balance']['id'],
-                "assetId": to_currency,
-                "value": str(Decimal(to_amount) * Decimal("-1"))
-            },
-            {
-                "bookId": self.bank_account['ledger_books']['blocked_balance']['id'],
-                "assetId": from_currency,
-                "value": str(Decimal(from_amount) * Decimal("-1"))
-            },
-            {
-                "bookId": self.DEFAULT_CASH_BOOK_ID['id'],
-                "assetId": to_currency,
-                "value": to_amount
+        block_book_order_balances = self.ledger.get_book_balances(
+            book_id=self.bank_account['ledger_books']['blocked_balance']['id'],
+            metadata_filter={
+                "order_id": order_id
             }
-        ]
-        metadata = {}
+        )
+
+        if block_book_order_balances == {}:
+            raise Exception(ErrorCodes.INSUFFICIENT_FUNDS.value)
+
         metadata['order_id'] = order_id
         metadata['order_type'] = order_type
         metadata['type'] = 'UNBLOCK'
         metadata['operation'] = 'ORDER'
-        operation = self.ledger.post_operation(operation_type, entries, memo, metadata)
+
+        entries = []
+        for asset, value in block_book_order_balances.items():
+            entries.extend([
+                {
+                    "bookId": self.bank_account['ledger_books']['blocked_balance']['id'],
+                    "assetId": asset,
+                    "value": str(Decimal(value) * Decimal("-1"))
+                },
+                {
+                    "bookId": self.bank_account['ledger_books']['main_balance']['id'],
+                    "assetId": asset,
+                    "value": str(Decimal(value))
+                }
+            ])
+
+        operation = self.ledger.post_operation(OperationType.CUSTOM, entries, memo, metadata)
         log.info(operation)
         return operation
 
-    def update_order_fill_status(self,order_id, order_type: str, from_currency: str, to_currency: str, from_amount: str, to_amount: str, memo: str, metadata: dict = {}):
+    def update_order_fill_status(self, order_id, order_type: str, from_currency: str, to_currency: str, from_amount: str, to_amount: str, memo: str, metadata: dict = {}):
         log.info("---------Update_Limit_Order_Funds----------")
 
-        if not memo:
-            raise Exception("Invalid memo")
-        memo = memo + f"_FILL_ORDER"
+        memo = f"{memo}_{order_id}_FILL_ORDER"
 
-        [main_book_order_balances, cash_book_order_balances, block_book_order_balances] = \
-            self.get_book_balances_bulk([{'book_id':self.bank_account['ledger_books']['main_balance']['id'], 'metadata_filter':{'order_id':order_id}},
-                                         {'book_id': self.DEFAULT_CASH_BOOK_ID['id'], 'metadata_filter': {'order_id': order_id}},
-                                         {'book_id':self.bank_account['ledger_books']['blocked_balance']['id'], 'metadata_filter':{'order_id':order_id}}])
-
-        if not block_book_order_balances:
-            raise Exception(ErrorCodes.INSUFFICIENT_FUNDS.value)
-        for (key,val) in block_book_order_balances.items():
-            if Decimal(val)<=0:
-                raise Exception(ErrorCodes.INSUFFICIENT_FUNDS.value)
-
-        if main_book_order_balances.get(to_currency,None) and cash_book_order_balances.get(from_currency,None):
-            if Decimal(to_amount) > Decimal(main_book_order_balances[to_currency]) and Decimal(from_amount) > Decimal(cash_book_order_balances[from_currency]):
-                to_amount = str(Decimal(to_amount) - Decimal(main_book_order_balances[to_currency]))
-                from_amount = str(Decimal(from_amount) - Decimal(cash_book_order_balances[from_currency]))
-            else:
-                raise Exception(ErrorCodes.FILL_REQUEST_SEQUENCE_ERROR.value)
-        else:
-            log.info(f'First fill request for order_id: {order_id} ,from_currency: {from_currency} ,from_amount: {from_amount} ,to_currency: {to_currency} ,to_amount: {to_amount}')
-        operation_type = OperationType.CUSTOM
-        entries = [
+        [main_book_order_balances, block_book_order_balances, cash_book_order_balances] = self.get_book_balances_bulk([
             {
-                "bookId": self.bank_account['ledger_books']['main_balance']['id'],
-                "assetId": to_currency,
-                "value": to_amount
+                'book_id': self.bank_account['ledger_books']['main_balance']['id'],
+                'metadata_filter': {'order_id': order_id}
             },
             {
-                "bookId": self.bank_account['ledger_books']['blocked_balance']['id'],
-                "assetId": to_currency,
-                "value": str(Decimal(to_amount) * Decimal("-1"))
+                'book_id': self.bank_account['ledger_books']['blocked_balance']['id'],
+                'metadata_filter':{'order_id':order_id}
             },
             {
-                "bookId": self.bank_account['ledger_books']['blocked_balance']['id'],
-                "assetId": from_currency,
-                "value": str(Decimal(from_amount) * Decimal("-1"))
-            },
-            {
-                "bookId": self.DEFAULT_CASH_BOOK_ID['id'],
-                "assetId": from_currency,
-                "value": from_amount
+                'book_id': self.DEFAULT_CASH_BOOK_ID['id'],
+                'metadata_filter': {'order_id': order_id}
             }
-        ]
+        ])
+
+        if block_book_order_balances == {}:
+            raise Exception(ErrorCodes.INSUFFICIENT_FUNDS.value)
+
+        from_currency_balance = cash_book_order_balances.get(from_currency, "0")
+        to_currency_balance = main_book_order_balances.get(to_currency, "0")
+
+        if Decimal(to_amount) < Decimal(to_currency_balance) or Decimal(from_amount) < Decimal(from_currency_balance):
+            raise Exception(ErrorCodes.FILL_REQUEST_SEQUENCE_ERROR.value)
+
+        delta_from_amount = str(Decimal(from_amount) - Decimal(from_currency_balance))
+        delta_to_amount = str(Decimal(to_amount) - Decimal(to_currency_balance))
+
+        if Decimal(delta_from_amount) == Decimal("0") and Decimal(delta_to_amount) == Decimal("0"):
+            return
+
+        if Decimal(delta_from_amount) > Decimal(block_book_order_balances.get(from_currency, "0")):
+            raise Exception(ErrorCodes.INSUFFICIENT_FUNDS.value)
 
-        metadata = {}
         metadata['order_id'] = order_id
         metadata['order_type'] = order_type
         metadata['type'] = 'FILL'
         metadata['operation'] = 'ORDER'
 
-        operation = self.ledger.post_operation(operation_type, entries, memo, metadata)
+        entries = [
+            {
+                "bookId": self.bank_account['ledger_books']['main_balance']['id'],
+                "assetId": to_currency,
+                "value": delta_to_amount
+            },
+            {
+                "bookId": self.bank_account['ledger_books']['blocked_balance']['id'],
+                "assetId": from_currency,
+                "value": str(Decimal(delta_from_amount) * Decimal("-1"))
+            },
+            {
+                "bookId": self.DEFAULT_CASH_BOOK_ID['id'],
+                "assetId": from_currency,
+                "value": delta_from_amount
+            },
+            {
+                "bookId": self.DEFAULT_CASH_BOOK_ID['id'],
+                "assetId": to_currency,
+                "value": str(Decimal(delta_to_amount) * Decimal("-1"))
+            }
+        ]
+
+        operation = self.ledger.post_operation(OperationType.CUSTOM, entries, memo, metadata)
         log.info(operation)
         return operation
 
